@@ -1,3 +1,12 @@
+///! The "public API" for the gc is the `Gc<T>` struct, which is a smart pointer to GCed data.
+///! Data can be "inserted" into the GC using `Gc::new()` for `Sized` types or `Gc::from_box()` for `?Sized` types.
+///! Data is accessed using `Deref`. 
+///! Mutable access is not supported, so mutable data that needs to be GCed must use interior mutability.
+///! As well, all GCed data must `impl GcTrace`.
+///! Weak references to GCed data can be obtained using `Gc::weakref()`.
+///! 
+///! `Gc<T>` supports a "thin pointer" representation and should not be wider than a single `usize`.
+
 use core::fmt;
 use core::ops::Deref;
 use core::borrow::Borrow;
@@ -7,12 +16,15 @@ use core::marker::PhantomData;
 use std::rc::Rc;
 
 use crate::runtime::gc::{GC_STATE, deref_safe};
-use crate::runtime::gc::data::{GcBox, GcTrace};
+use crate::runtime::gc::trace::GcTrace;
+use crate::runtime::gc::gcbox::{GcBox, GcBoxPtr};
+use crate::runtime::gc::ptrmeta::PtrMetadata;
 use crate::runtime::gc::weak::GcWeakCell;
 
 
+///! Smart pointer to GCed data. See the module-level documentation for more details.
 pub struct Gc<T> where T: GcTrace + ?Sized + 'static {
-    ptr: NonNull<GcBox<T>>,
+    ptr: GcBoxPtr,
     _marker: PhantomData<Rc<GcBox<T>>>,
 }
 
@@ -30,6 +42,7 @@ impl<T: GcTrace> Gc<T> {
 
 impl<T> Gc<T> where 
     T: GcTrace + ?Sized + Pointee, 
+    T::Metadata: Into<PtrMetadata>,
     GcBox<T>: Pointee<Metadata = T::Metadata> 
 {
     pub fn from_box(data: Box<T>) -> Self {
@@ -44,31 +57,11 @@ impl<T> Gc<T> where
 }
 
 impl<T> Gc<T> where T: GcTrace + ?Sized {
-    
     pub(super) fn from_raw(ptr: NonNull<GcBox<T>>) -> Self {
-        Self { ptr, _marker: PhantomData }
-    }
-    
-    #[inline]
-    pub(super) fn inner(&self) -> &GcBox<T> {
-        // must not deref during sweep. This should only be possible if called inside a Drop impl
-        debug_assert!(deref_safe());
-        unsafe { self.ptr.as_ref() }
-    }
-    
-    fn inner_mut(&mut self) -> &mut GcBox<T> {
-        debug_assert!(deref_safe());
-        unsafe { self.ptr.as_mut() }
-    }
-    
-    pub fn mark_trace(mut self) {
-        self.inner_mut().mark_trace()
-    }
-    
-    /// Create a weak reference from this GC handle
-    pub fn weakref(&self) -> GcWeak<T> {
-        let weak_ptr = GcBox::get_or_make_weak(self.ptr);
-        GcWeak::new(Gc::from_raw(weak_ptr))
+        Self { 
+            ptr: ptr.into(),
+            _marker: PhantomData,
+        }
     }
     
     pub fn ptr_eq<U>(self_gc: &Gc<T>, other_gc: &Gc<U>) -> bool where U: GcTrace + ?Sized {
@@ -83,7 +76,34 @@ impl<T> Gc<T> where T: GcTrace + ?Sized {
     pub fn as_id(self_gc: &Gc<T>) -> usize {
         self_gc.ptr.as_ptr() as *const () as usize
     }
+}
 
+impl<T> Gc<T> where 
+    T: GcTrace + ?Sized, 
+    PtrMetadata: TryInto<<GcBox<T> as Pointee>::Metadata>
+{
+    #[inline]
+    fn inner(&self) -> &GcBox<T> {
+        // must not deref during sweep. This should only be possible if called inside a Drop impl
+        debug_assert!(deref_safe());
+        unsafe { self.ptr.to_gcbox_ptr().as_ref() }
+    }
+    
+    #[inline]
+    fn inner_mut(&mut self) -> &mut GcBox<T> {
+        debug_assert!(deref_safe());
+        unsafe { self.ptr.to_gcbox_ptr().as_mut() }
+    }
+    
+    /// Create a weak reference from this GC handle
+    pub fn weakref(&self) -> GcWeak<T> {
+        let weak_ptr = GcBox::get_or_make_weak(self.ptr.to_gcbox_ptr());
+        GcWeak::new(Gc::from_raw(weak_ptr))
+    }
+    
+    pub fn mark_trace(mut self) {
+        self.inner_mut().mark_trace()
+    }
 }
 
 impl<T> From<Gc<T>> for Gc<dyn GcTrace> where T: GcTrace {
@@ -95,19 +115,28 @@ impl<T> From<Gc<T>> for Gc<dyn GcTrace> where T: GcTrace {
     }
 }
 
-impl<T> AsRef<T> for Gc<T> where T: GcTrace + ?Sized {
+impl<T> AsRef<T> for Gc<T> where 
+    T: GcTrace + ?Sized,
+    PtrMetadata: TryInto<<GcBox<T> as Pointee>::Metadata>
+{
     fn as_ref(&self) -> &T {
         self.deref()
     }
 }
 
-impl<T> Borrow<T> for Gc<T> where T: GcTrace + ?Sized {
+impl<T> Borrow<T> for Gc<T> where 
+    T: GcTrace + ?Sized,
+    PtrMetadata: TryInto<<GcBox<T> as Pointee>::Metadata>
+{
     fn borrow(&self) -> &T {
         self.deref()
     }
 }
 
-impl<T> Deref for Gc<T> where T: GcTrace + ?Sized {
+impl<T> Deref for Gc<T> where 
+    T: GcTrace + ?Sized,
+    PtrMetadata: TryInto<<GcBox<T> as Pointee>::Metadata>
+{
     type Target = T;
     
     #[inline]
@@ -127,9 +156,9 @@ impl<T> Clone for Gc<T> where T: GcTrace + ?Sized {
 
 impl<T> Copy for Gc<T> where T: GcTrace + ?Sized { }
 
-impl<T> Hash for Gc<T> where T: GcTrace {
+impl<T> Hash for Gc<T> where T: GcTrace + ?Sized {
     fn hash<H>(&self, state: &mut H) where H: Hasher {
-        <NonNull<GcBox<T>> as Hash>::hash(&self.ptr, state)
+        self.ptr.hash(state)
     }
 }
 
@@ -142,7 +171,6 @@ impl<T> fmt::Debug for Gc<T> where T: GcTrace + ?Sized {
 }
 
 
-#[derive(Clone, Copy)]
 pub struct GcWeak<T> where T: GcTrace + ?Sized + 'static {
     gc_weak: Gc<GcWeakCell<T>>,
 }
@@ -180,6 +208,16 @@ impl<T> GcWeak<T> where T: GcTrace + ?Sized {
     }
 }
 
+impl<T> Clone for GcWeak<T> where T: GcTrace + ?Sized {
+    fn clone(&self) -> Self {
+        Self {
+            gc_weak: self.gc_weak.clone()
+        }
+    }
+}
+
+impl<T> Copy for GcWeak<T> where T: GcTrace + ?Sized { }
+
 impl<T> fmt::Debug for GcWeak<T> where T: GcTrace + ?Sized {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt.debug_tuple("GcWeak")
@@ -194,6 +232,12 @@ mod tests {
     use super::*;
     use core::cell::Cell;
     use crate::runtime::gc::gc_force;
+    use static_assertions::assert_eq_size;
+    
+    assert_eq_size!(Gc<i32>, usize);
+    assert_eq_size!(Gc<[i32]>, usize);
+    
+    // comment this out when using miri
     // use test_log::test;
     
     #[test]
